﻿#region Revision info

/*
 * $Author: Apoc $
 * $Date$
 * $ID$
 * $Revision$
 * $URL$
 * $LastChangedBy: wulf $
 * $ChangesMade$
 */

#endregion Revision info

using System;
using System.Collections.Generic;
using System.Linq;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace MarkII.Misc
{
    /// <summary>
    /// Calculates DPS, and combat time left on nearby mobs in combat.
    /// DPS is not calculated as YOUR DPS. But instead; global DPS on the mob. (You and your buddies combined DPS on a single mob => RaidDPS)
    /// </summary>
    internal static class DpsMeter
    {
        private static readonly Dictionary<ulong, DpsInfo> DpsInfos = new Dictionary<ulong, DpsInfo>();
        public static bool DpsMeterInitialized;

        /// <summary>
        /// Starts the DpsMeter.
        /// </summary>
        public static void Initialize()
        {
            if (DpsMeterInitialized) return;
            DpsInfos.Clear();
            DpsMeterInitialized = true;
        }

        public static void Shutdown()
        {
            if (!DpsMeterInitialized) return;
            DpsInfos.Clear();
            DpsMeterInitialized = false;
        }

        public static void Update()
        {
            List<WoWUnit> availableUnits = (from u in ObjectManager.GetObjectsOfType<WoWUnit>(true, false)
                                            where !u.IsDead && !u.IsFriendly && u.IsTargetingUs() && u.Combat && u.HealthPercent < 100
                                            orderby u.DistanceSqr ascending
                                            select u).ToList();
            //  Logger.DebugLog("Dpsmeter availableUnits: {0}!", availableUnits.Count);

            foreach (WoWUnit unit in availableUnits)
            {
                if (!DpsInfos.ContainsKey(unit.Guid) && availableUnits.Count > 0)
                {
                    var di = new DpsInfo { Unit = unit, CombatStart = DateTime.Now, StartHealth = unit.CurrentHealth };

                    DpsInfos.Add(unit.Guid, di);
                }
                else if (availableUnits.Count > 0)
                {
                    DpsInfo di = DpsInfos[unit.Guid];

                    // Let's calculate the Δdeltas for a cleaner Math!
                    uint dhealth = di.StartHealth - unit.CurrentHealth;
                    TimeSpan dtime = DateTime.Now - di.CombatStart;

                    double timeLeft = unit.CurrentHealth / di.CurrentDps;
                    di.CurrentDps = dhealth / dtime.TotalSeconds;
                    di.CombatTimeLeft = new TimeSpan(0, 0, (int)timeLeft);

                    // .NET makes a copy of the struct when we grab it out of the collection.
                    // Make sure we put the updated version back in!
                    DpsInfos[unit.Guid] = di;
                  //   Logger.DebugLog("Dpsmeter ticking! {0} DPS/{1}s ({2})", di.CurrentDps, di.CombatTimeLeft, dtime.TotalSeconds); // Just for observation purposes, can get removed when working!

                }
            }

            // Kill off any 'bad' units in our list.
            KeyValuePair<ulong, DpsInfo>[] removeUnits = DpsInfos.Where(kv => !kv.Value.Unit.IsValid || kv.Value.Unit.IsDead || !kv.Value.Unit.IsTargetingUs() || kv.Value.Unit.IsFriendly || kv.Value.Unit.HealthPercent >= 100 || kv.Value.Unit == null).ToArray();

            foreach (KeyValuePair<ulong, DpsInfo> t in removeUnits)
            {
                DpsInfos.Remove(t.Key);
                 // Logger.DebugLog("Dpsmeter removed bad units"); // Just for observation purposes, can get removed when working!
            }
        }

        /// <summary>
        /// Returns the current DPS on a specific unit, or -1 if the unit is not currently being tracked, or doesn't exist.
        /// </summary>
        /// <param name="u"></param>
        /// <returns></returns>
        public static double GetDps(WoWUnit u)
        {
            // -1 is a fail case.
            return DpsInfos.ContainsKey(u.Guid) ? DpsInfos[u.Guid].CurrentDps : -1;
        }

        /// <summary>
        /// Returns the estimated combat time left for this unit. (Time until death)
        /// If the unit is invalid; TimeSpan.MinValue is returned.
        /// </summary>
        /// <param name="u"></param>
        /// <param name="time"></param>
        /// <returns></returns>
        public static TimeSpan GetCombatTimeLeft(WoWUnit u, int time)
        {
            //If our current target is a dummy, we set a fixed value for combat time left (55).
            if (Styx.StyxWoW.Me.GotTarget && Lists.TrainingDummies.Contains(Styx.StyxWoW.Me.CurrentTarget.Entry))
                return new TimeSpan(0, 0, 61);

            if (DpsInfos.ContainsKey(u.Guid))
            {
                TimeSpan dtime = DateTime.Now - DpsInfos[u.Guid].CombatStart;

                return DpsInfos.ContainsKey(u.Guid) && dtime.TotalSeconds > time ? DpsInfos[u.Guid].CombatTimeLeft : TimeSpan.MinValue;
            }
            return TimeSpan.MinValue;
        }

        #region Nested type: DpsInfo

        private struct DpsInfo
        {
            public DateTime CombatStart;
            public TimeSpan CombatTimeLeft;
            public double CurrentDps;
            public uint StartHealth;
            public WoWUnit Unit;
        }

        #endregion
    }
}